/*
 * onewire.c
 *
 *  Created on: 13.02.2012
 *      Author: di
 */

#include "onewire.h"


#define OW_DMA_CH_RX 	DMA1_Channel3
#define OW_DMA_CH_TX 	DMA1_Channel2
#define OW_DMA_FLAG		DMA1_FLAG_TC3



//   /  1-wire
uint8_t ow_buf[8];

#define OW_0	0x00
#define OW_1	0xff
#define OW_R_1	0xff

#define OW_TimeOut 100 //      .

//-----------------------------------------------------------------------------
//      ,    USART
// ow_byte - ,   
// ow_bits -   ,    8 
//-----------------------------------------------------------------------------
void OW_toBits(uint8_t ow_byte, uint8_t *ow_bits)
{
    uint8_t i;
    for (i = 0; i < 8; i++)
    {
        if (ow_byte & 0x01)
        {
            *ow_bits = OW_1;
        }
        else
        {
            *ow_bits = OW_0;
        }
        ow_bits++;
        ow_byte = ow_byte >> 1;
    }
}

//-----------------------------------------------------------------------------
//   -  ,    USART   
// ow_bits -   ,    8 
//-----------------------------------------------------------------------------
uint8_t OW_toByte(uint8_t *ow_bits)
{
    uint8_t ow_byte, i;
    ow_byte = 0;
    for (i = 0; i < 8; i++)
    {
        ow_byte = ow_byte >> 1;
        if (*ow_bits == OW_R_1)
        {
            ow_byte |= 0x80;
        }
        ow_bits++;
    }

    return ow_byte;
}

//-----------------------------------------------------------------------------
//  USART  DMA
//-----------------------------------------------------------------------------
uint8_t OW_Init()
{
    GPIO_InitTypeDef GPIO_InitStruct;
    USART_InitTypeDef USART_InitStructure;

    RCC_USARTCLKConfig(RCC_USART1CLK_SYSCLK);  // USART    .

    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);    //    GPIOA,
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE); // USART1,
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);     //  DMA1.

    // USART TX
    GPIO_InitStruct.GPIO_Pin = GPIO_Pin_2;        //   PA.2.
    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;     //   .
    GPIO_InitStruct.GPIO_OType = GPIO_OType_OD;   //    .
    GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL; //   .
    GPIO_InitStruct.GPIO_Speed = GPIO_Speed_10MHz;

    GPIO_Init(GPIOA, &GPIO_InitStruct);

    GPIO_PinAFConfig(GPIOA, GPIO_PinSource2, GPIO_AF_1); // GPIOA.2   USART1.

    //  USART.
    USART_InitStructure.USART_BaudRate = 115200;                //  .
    USART_InitStructure.USART_WordLength = USART_WordLength_8b; // 8  .
    USART_InitStructure.USART_StopBits = USART_StopBits_1;      // 1  .
    USART_InitStructure.USART_Parity = USART_Parity_No;         //   .
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode = USART_Mode_Tx | USART_Mode_Rx; //    .

    USART_Init(USART1, &USART_InitStructure);

    USART_SetAddress(USART1, 0);          // .      .
    USART_HalfDuplexCmd(USART1 , ENABLE); //   USART   .
    USART_Cmd(USART1, ENABLE);            //  USART.

    return OW_OK;
}

//-----------------------------------------------------------------------------
//         
//-----------------------------------------------------------------------------
uint8_t OW_Reset()
{
    uint32_t times;
    uint8_t ow_presence;
    USART_InitTypeDef USART_InitStructure;

    //  USART.
    USART_InitStructure.USART_BaudRate = 9600;
    USART_InitStructure.USART_WordLength = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits = USART_StopBits_1;
    USART_InitStructure.USART_Parity = USART_Parity_No;
    USART_InitStructure.USART_HardwareFlowControl =
        USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode = USART_Mode_Tx | USART_Mode_Rx;
    USART_Init(USART1, &USART_InitStructure);
    USART_Cmd(USART1, ENABLE);

    //  0xf0   9600
    USART_ClearFlag(USART1, USART_FLAG_TC);
    USART_SendData(USART1, 0xf0);

    times = SysTim_GetTick(); //        .

    while (USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET)
    {

        if (SysTim_GetTick()-times > OW_TimeOut)
        {
            return OW_Err_TimeOut; //  .
        }

#ifdef OW_GIVE_TICK_RTOS
        taskYIELD();
#endif
    }

    while (USART_GetFlagStatus(USART1, USART_FLAG_RXNE) == RESET)
    {

        if (SysTim_GetTick()-times > OW_TimeOut)
        {
            return OW_Err_TimeOut; //  .
        }

#ifdef OW_GIVE_TICK_RTOS
        taskYIELD();
#endif
    }

    ow_presence = USART_ReceiveData(USART1); //  .

    //  USART.
    USART_InitStructure.USART_BaudRate = 115200;
    USART_Init(USART1, &USART_InitStructure);
    USART_Cmd(USART1, ENABLE);

    if (ow_presence != 0xf0 || ow_presence != 0 || ow_presence != 0xFF)
    {
        return OW_OK;
    }

    return OW_NO_DEVICE;
}


//-----------------------------------------------------------------------------
//     1-wire
// sendReset -  RESET   .
// 		OW_SEND_RESET  OW_NO_RESET
// command -  ,   .    -  OW_READ_SLOTH
// cLen -   ,     
// data -   ,      
// dLen -    .     
// readStart -       (  0)
//		  OW_NO_READ,     data  dLen
//-----------------------------------------------------------------------------
uint8_t OW_Send(uint8_t sendReset, uint8_t *command, uint8_t cLen,
                uint8_t *data, uint8_t dLen, uint8_t readStart)
{
    uint32_t times;
    uint8_t ret;
    //    -      
    if (sendReset == OW_SEND_RESET)
    {
        ret = OW_Reset();
        if (ret != OW_OK)
        {
            return ret;
        }
    }

    while (cLen > 0)
    {

        OW_toBits(*command, ow_buf);
        command++;
        cLen--;

        DMA_InitTypeDef DMA_InitStructure;

        // DMA  
        DMA_DeInit(OW_DMA_CH_RX);
        DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t) &(USART1->RDR); //  -    USART.
        DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t) &ow_buf[0];         //  -   .
        DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC; //      .
        DMA_InitStructure.DMA_BufferSize = 8; //     .
        DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable; //    .
        DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;          //    (   ).
        DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte; //    1 .
        DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;  //   1 .
        DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;      //    .
        DMA_InitStructure.DMA_Priority = DMA_Priority_Low; //  .
        DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
        DMA_Init(OW_DMA_CH_RX, &DMA_InitStructure);

        // DMA  
        DMA_DeInit(OW_DMA_CH_TX);
        DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t) &(USART1->TDR); //  -    USART.
        DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST; //      .
        DMA_Init(OW_DMA_CH_TX, &DMA_InitStructure);

        //   
        USART_ClearFlag(USART1, USART_FLAG_RXNE | USART_FLAG_TC | USART_FLAG_TXE); //   USATR

        USART_DMACmd(USART1, USART_DMAReq_Tx | USART_DMAReq_Rx, ENABLE); //  DMA  USART.
        DMA_Cmd(OW_DMA_CH_RX, ENABLE); //  DMA.
        DMA_Cmd(OW_DMA_CH_TX, ENABLE);

        times = SysTim_GetTick(); //        .

        // ,    8 
        while (DMA_GetFlagStatus(OW_DMA_FLAG) == RESET)
        {

            if (SysTim_GetTick()-times > OW_TimeOut) //  .
            {
                //  DMA
                DMA_Cmd(OW_DMA_CH_TX, DISABLE);
                DMA_Cmd(OW_DMA_CH_RX, DISABLE);
                USART_DMACmd(USART1, USART_DMAReq_Tx | USART_DMAReq_Rx, DISABLE);
                return OW_Err_TimeOut;
            }

#ifdef OW_GIVE_TICK_RTOS
            taskYIELD();
#endif
        }

        //  DMA
        DMA_Cmd(OW_DMA_CH_TX, DISABLE);
        DMA_Cmd(OW_DMA_CH_RX, DISABLE);
        USART_DMACmd(USART1, USART_DMAReq_Tx | USART_DMAReq_Rx, DISABLE);

        //    -  -    
        if (readStart == 0 && dLen > 0)
        {
            *data = OW_toByte(ow_buf);
            data++;
            dLen--;
        }
        else
        {
            if (readStart != OW_NO_READ)
            {
                readStart--;
            }
        }
    }

    return OW_OK;
}

uint8_t OW_CRC8(uint8_t *Buff, uint8_t Len)
{
  uint8_t i; uint8_t x; uint8_t b; uint8_t crc=0;

  for (i=0; i<Len; i++)
  {
    b = Buff[i];
    for (x=0; x<8; x++)
    {
        if ((b ^ crc) & 1)
        {
            crc = ((crc^0x18)>>1)|0x80;
        }
        else
        {
            crc = crc >> 1;
        }
        b = b >> 1;
    }
  }

  return crc;
}


void OW_out_set_as_TX_pin(void)
{
    GPIO_InitTypeDef GPIO_InitStruct;

    // USART TX
    GPIO_InitStruct.GPIO_Pin = GPIO_Pin_2;
    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStruct.GPIO_OType = GPIO_OType_OD;
    GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStruct);

    GPIO_PinAFConfig(GPIOA, GPIO_PinSource2, GPIO_AF_1); // GPIOA.2   USART1.
}

void OW_out_set_as_Power_pin(void)
{
    GPIO_InitTypeDef GPIO_InitStruct;

    // GPIO
    GPIO_SetBits(GPIOA, GPIO_Pin_2); // . 1   PA.2.
    GPIO_InitStruct.GPIO_Pin = GPIO_Pin_2;
    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStruct);
}

